/**
  *  \file client/widgets/commandinfo.cc
  *  \brief WCommandInfo widget
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/widgets/commandinfo.h"
#include "game/cmds.h"
#include "util/nls.h"
#include "ui/uiglobal.h"
#include "ui/util.h"

/** Constructor. */
WCommandInfo::WCommandInfo()
    : cmd(0)
{ }

/** Destructor. */
WCommandInfo::~WCommandInfo()
{ }

/** Set command to display. Can be null. */
void
WCommandInfo::setCommand(const GCommand* cmd)
{
    this->cmd = cmd;
    drawWidget();
}

void
WCommandInfo::getLayoutInfo(LayoutInfo& info)
{
    info.pref_size = info.min_size = font_sizes[FONT_NORMAL].scaledBy(30, 2);
    info.flags = li_FlexibleH;
}

/** Command information strings. Indexed by GCommand::Type. */
static const char*const command_info[] = {
    N_("Change message language"),
    N_("Request configuration"),
    N_("Request race names file"),
    N_("Change our race name"),
    N_("Change message detail (host-side filter)"),
    N_("Change ally privileges"),
    N_("Add/drop ally (PHost version)"),
    N_("Give away a ship"),
    N_("Give away a planet"),
    N_("Configure or request remote control over a ship"),
    N_("Configure remote control setting for newly-built ships"),
    N_("Gather resources from a planet"),
    N_("Add/drop ally (Host 3.22.007+ version)"),
    N_("Request friendly code list"),
    N_("Send file with TRN (will echo back with next RST, PHost 4.0+)"),
    N_("Add/drop permanent enemy"),
    N_("Unload cash/ammo onto foreign planet"),
    N_("Transfer cash/ammo to foreign ship"),
    N_("Show ship to an ally"),
    N_("Show planet to an ally"),
    N_("Show minefield to an ally"),
    N_("Configure parts for Super-Refit mission"),
};
CTASSERT(countof(command_info) == GCommand::phc_Other);

void
WCommandInfo::drawContent(GfxCanvas& can)
{
    // Figure out what to display
    string_t text;
    if (cmd != 0) {
        const GCommand::Type t = cmd->getCommand();
        if (t >= GCommand::phc_Other) {
            text = _("(command not understood by PCC2)");
        } else {
            text = _(command_info[t]);
        }
    }

    // Display it
    UISkin& sk = getSkin();
    sk.startDraw();
    sk.drawBackground(can, getExtent());
    sk.setColor(UIColor::tc_Faded);
    sk.setSolid(false);
    sk.getParameters().font = fonts[FONT_NORMAL];
    outTextFormatted(can, sk, getExtent().x, getExtent().y, getExtent().w, text);

    can.update(getExtent());
}
