/**
  *  \file gfx/pixmap.h
  *  \brief Pixmaps
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GFX_PIXMAP_H
#define PCC_V2_GFX_PIXMAP_H

#include "gfx/gfx.h"
#include "io/streams.h"

class GfxPixmap : public GfxSurface {
    void copySettingsFrom(const GfxPixmap& other);
    Ptr<GfxPixmap> convertTo(GfxSystemPixelFormat format) const;
 public:
    GfxPixmap(int w, int h, int depth, uint32 rmask, uint32 gmask, uint32 bmask, uint32 amask);
    GfxPixmap(SDL_Surface* sfc, bool aown);
    GfxPixmap(int w, int h, GfxSurface& copy_format);

    void setFormat(int w, int h, int depth, uint32 rmask, uint32 gmask, uint32 bmask, uint32 amask);

    Ptr<GfxPixmap> convertToScreenFormat() const;

    void blit(GfxCanvas& can, int x, int y);
    void blit(GfxCanvas& can, int x, int y, GfxRect subr);
    void blitSized(GfxCanvas& can, int x, int y, int wi, int he);

    void tile(GfxCanvas& can, const GfxRect& area, int alt=0);
    void tileAnchored(GfxCanvas& can, const GfxRect& area, int ax=0, int ay=0, int alt=0);

    void flipHorizontal();
    void flipVertical();

    /** Get width of this pixmap. */
    int getWidth() const
        { return sfc->w; }
    /** Get height of this pixmap. */
    int getHeight() const
        { return sfc->h; }

    /** Check whether this pixmap is colorkeyed. */
    bool isColorKeyed() const
        { return getColorKey() == GFX_TRANSPARENT; }
    uint32 getColorKey() const;
    void setColorKey(uint32 color);

    uint8 getBlitAlpha() const;
    void  setBlitAlpha(uint8 alpha);
    bool  isBlitAlphaSupported() const;

    GfxRect computeBoundingRectangle(uint32 color);

    Ptr<GfxPixmap> clone() const;
    Ptr<GfxPixmap> cloneCropped(uint32 color);
};

Ptr<GfxPixmap> loadImage(Ptr<Stream> s);

#endif
