/**
  *  \file u/t_game_playerset.cc
  */

#include "game/playerset.h"

#include "u/t_game.h"

void
GamePlayerSetTestSuite::testMatrix()
{
    GPlayerBitMatrix mtx;

    // check zero-initialisation
    for (int i = 1; i <= 12; ++i)
        for (int j = 1; j <= 12; ++j)
            TS_ASSERT(!mtx.get(i, j));
    for (int i = 1; i <= 12; ++i)
        TS_ASSERT(mtx.getRow(i).empty());

    // set some value and check that surroundings are not modified. Do so twice.
    for (int i = 0; i < 2; ++i) {
        mtx.set(3, 7, true);
        TS_ASSERT(!mtx.get(2, 6));
        TS_ASSERT(!mtx.get(2, 7));
        TS_ASSERT(!mtx.get(2, 8));
        TS_ASSERT(!mtx.get(3, 6));
        TS_ASSERT( mtx.get(3, 7));
        TS_ASSERT(!mtx.get(3, 8));
        TS_ASSERT(!mtx.get(4, 6));
        TS_ASSERT(!mtx.get(4, 7));
        TS_ASSERT(!mtx.get(4, 8));
        TS_ASSERT(mtx.getRow(2).empty());
        TS_ASSERT_EQUALS(mtx.getRow(3), GPlayerSet(7));
        TS_ASSERT(mtx.getRow(4).empty());
    }

    // set neighbouring bit (same row)
    for (int i = 0; i < 2; ++i) {
        mtx.set(3, 8, true);
        TS_ASSERT(!mtx.get(2, 7));
        TS_ASSERT(!mtx.get(2, 8));
        TS_ASSERT(!mtx.get(2, 9));
        TS_ASSERT( mtx.get(3, 7));
        TS_ASSERT( mtx.get(3, 8));
        TS_ASSERT(!mtx.get(3, 9));
        TS_ASSERT(!mtx.get(4, 7));
        TS_ASSERT(!mtx.get(4, 8));
        TS_ASSERT(!mtx.get(4, 9));
        TS_ASSERT(mtx.getRow(2).empty());
        TS_ASSERT_EQUALS(mtx.getRow(3), GPlayerSet(7) | GPlayerSet(8));
        TS_ASSERT(mtx.getRow(4).empty());
    }

    // set neighbouring bit (different row)
    for (int i = 0; i < 2; ++i) {
        mtx.set(2, 6, true);
        TS_ASSERT(!mtx.get(1, 5));
        TS_ASSERT(!mtx.get(1, 6));
        TS_ASSERT(!mtx.get(1, 7));
        TS_ASSERT(!mtx.get(2, 5));
        TS_ASSERT( mtx.get(2, 6));
        TS_ASSERT(!mtx.get(2, 7));
        TS_ASSERT(!mtx.get(3, 5));
        TS_ASSERT(!mtx.get(3, 6));
        TS_ASSERT( mtx.get(3, 7));
        TS_ASSERT(mtx.getRow(1).empty());
        TS_ASSERT_EQUALS(mtx.getRow(2), GPlayerSet(6));
        TS_ASSERT_EQUALS(mtx.getRow(3), GPlayerSet(7) | GPlayerSet(8));
    }

    // check some out-of-range positions
    TS_ASSERT(!mtx.get(99, 2));
    TS_ASSERT(!mtx.get(99, 99));
    TS_ASSERT(!mtx.get(2, 99));
    TS_ASSERT(!mtx.get(-99, 99));
    TS_ASSERT(!mtx.get(1, 13));
    TS_ASSERT(!mtx.get(13, 1));

    // clear it again and check zeroness again
    mtx.clear();

    for (int i = 1; i <= 12; ++i)
        for (int j = 1; j <= 12; ++j)
            TS_ASSERT(!mtx.get(i, j));
    for (int i = 1; i <= 12; ++i)
        TS_ASSERT(mtx.getRow(i).empty());
}

void
GamePlayerSetTestSuite::testArray()
{
    GPlayerArray<int> n;

    // check indexing
    TS_ASSERT(!n.isValidIndex(-2));
    TS_ASSERT(!n.isValidIndex(-1));
    TS_ASSERT(n.isValidIndex(0));
    TS_ASSERT(n.isValidIndex(1));
    TS_ASSERT(n.isValidIndex(10));
    TS_ASSERT(n.isValidIndex(11));
    TS_ASSERT(n.isValidIndex(12));
    TS_ASSERT(!n.isValidIndex(-13));
    TS_ASSERT(!n.isValidIndex(-14));

    // check initialisation
    n.setAll(0);
    TS_ASSERT_EQUALS(n[0], 0);
    TS_ASSERT_EQUALS(n[1], 0);
    TS_ASSERT_EQUALS(n[2], 0);
    TS_ASSERT_EQUALS(n[10], 0);
    TS_ASSERT_EQUALS(n[11], 0);
    TS_ASSERT_EQUALS(n[12], 0);

    // check initialisation
    n.setAll(42);
    TS_ASSERT_EQUALS(n[0], 42);
    TS_ASSERT_EQUALS(n[1], 42);
    TS_ASSERT_EQUALS(n[2], 42);
    TS_ASSERT_EQUALS(n[10], 42);
    TS_ASSERT_EQUALS(n[11], 42);
    TS_ASSERT_EQUALS(n[12], 42);

    // check assignment
    n[2] = 8;
    TS_ASSERT_EQUALS(n[0], 42);
    TS_ASSERT_EQUALS(n[1], 42);
    TS_ASSERT_EQUALS(n[2], 8);
    TS_ASSERT_EQUALS(n[3], 42);
    TS_ASSERT_EQUALS(n[4], 42);

    // check modify-assignment
    n[2] += 7;
    TS_ASSERT_EQUALS(n[0], 42);
    TS_ASSERT_EQUALS(n[1], 42);
    TS_ASSERT_EQUALS(n[2], 15);
    TS_ASSERT_EQUALS(n[3], 42);
    TS_ASSERT_EQUALS(n[4], 42);
}
